// node modules
import React, {
    useState,
    useRef,
    FocusEvent,
    ChangeEvent,
    FC,
    ReactNode,
} from 'react';

// local components
import { Typography } from "../typography";

// local files
import clearIconSvg from '../../assets/clear.svg';
import { useStyles } from './styles';
import { theming } from '../../theme';

export interface InputProps {
    labelClassName: string;
    rootClassName: string;
    inputClassName: string;
    onChange: (e: ChangeEvent | Event)=> void;
    value: string | number;
    label?: string;
    disabled: boolean;
    onFocus?: (e: FocusEvent) => void;
    onBlur?: (e: FocusEvent) => void;
    type?: string;
    error?: boolean;
    adornmentStart?: string | ReactNode;
    adornmentEnd?: string | ReactNode;
    closeIcon?: boolean;
}

export const Input: FC<InputProps> = ({
    error= false,
    type= 'text',
    disabled = false,
    labelClassName,
    rootClassName,
    inputClassName,
    onChange,
    label,
    value,
    onFocus,
    onBlur,
    adornmentEnd,
    adornmentStart,
    closeIcon,
    ...otherProps
}) => {
    const {
        root,
        labelClass,
        active,
        input,
        unlabeled,
        disabledClass,
        errorClass,
        typeWrapper,
        clearIconClass,
        inputAdorned,
        inputAdornedActive
    } = useStyles({ theme: theming.useTheme() });
    const inputEl = useRef({} as HTMLInputElement);
    const [activeClass, setActiveClass] = useState(false);
    const [inputValue, setInputValue] = useState(value || '');
    const handleFocus = (e: FocusEvent<HTMLInputElement>) => {
        if(disabled) return;

        setActiveClass(true);

        if (onFocus) {
            onFocus(e)
        }
    }
    const handleBlur = (e: FocusEvent<HTMLInputElement>) => {
        setActiveClass(!!e.target.value)

        if(onBlur) {
            onBlur(e)
        }
    }
    const handleChange = (e: ChangeEvent<HTMLInputElement>) => {
        setInputValue(e.target.value);

        if (onChange) {
            onChange(e)
        }
    }
    const handleClean = () => {
        const evt = new Event('change');
        const { current } = inputEl;

        setInputValue('');

        Object.defineProperty(evt, 'target',{ writable: false, value: current });
        Object.defineProperty(evt, 'currentTarget',{ writable: false, value: current });

        if (onChange) {
            onChange(evt);
        }

        current.focus();
    }

    let adornmentClass = '';

    if (adornmentStart) {
        adornmentClass = `${inputAdorned} ${!!inputValue || activeClass ? inputAdornedActive : ''}`;
    }

    const classNameLabel = `${labelClass} ${labelClassName} ${!!inputValue || activeClass ? active : ''} ${adornmentClass}`
    const rootClass = `${root} ${rootClassName} ${disabled ? disabledClass: ''} ${error ? errorClass : '' }`
    const inputClass = `${input} ${!label ? unlabeled : ''}`

    return (
        <div className={rootClass}>
            {adornmentStart && (
                <div className={typeWrapper}>
                    <Typography variant="body1">
                        {adornmentStart}
                    </Typography>
                </div>
            )}
            {label && <span className={classNameLabel}>{label}</span>}
            <input
                ref={inputEl}
                type={type}
                disabled={disabled}
                onBlur={handleBlur}
                onFocus={handleFocus}
                className={inputClass}
                value={inputValue}
                onChange={handleChange}
                {...otherProps}
            />
            {adornmentEnd && (
                <div className={typeWrapper}>
                    <Typography variant="body1">
                        {adornmentEnd}
                    </Typography>
                </div>
            )}
            {closeIcon && (
                <img className={clearIconClass} onClick={handleClean} src={clearIconSvg} alt=""/>
            )}
        </div>
    )
}






